
;; Globals. Idlemax count the number of ticks we let pass before
;; sending out an idle message.  Confused is the number of consecutive
;; help requests we let pass before assuming the student is
;; in the confused state and needs help.

(setq ifd-varlist '((tick . 0)(idlemax . 10)(confused . 3)
		    (student . nil)(name . "")(lab . |#15277|)))

;; Macros, with no arguments.  These just result in a substitution of
;; (f) by its body.

(cleardefs)

(defaction start-student-record
  (do (robot-mail-action "~a: Starting to track student." name)
      (comment "~%Should probably direct log of student actions to a file.~%")))

(defaction save-student-record
  (do (robot-mail-action "~a: Ceasing to track student." name)
      (comment "~%Need to save student action log file.~%")))

;; (nodename [type] [children] [enter] [exit] [initial])

(setq ifd-nodes
      '((student or (out check in) nil nil out)
	(out basic)
	(check basic)
	(in and (moo tutorial))
	(moo or (active idle confused help) nil nil active)
	(active basic nil (assign tick 0))
	;; (exploring basic)
	;; (tutoring basic)
	(help basic)
	(idle basic)
	(confused basic)
	(tutorial or (neutral working) nil nil neutral)
	(neutral)
	(working or (create setup run checkresult) nil nil create)
	(create)
	(setup)
	(run)
	(checkresult)))

(setq ifd-events nil) 

;; (arcname from to [trigger] [action])

(setq ifd-arcs
      '((a1  out  check
	     (notify (connect (player . (? student))))
	     (assign student student))	;; student is a local and not preserved unless we do this.
	(a20 out  check
	     (notify (moveto (player . (? student)) (to . (@ lab))))
	     (assign student student))

; 	(a16 out  in
; 	     (cond nil (robot-find-student))
; 	     (do (assign student (robot-find-student))
; 		 (assign name (robot-whois (robot-find-student)))))	

	(a14 check out
	     (cond nil (not (studentp student)))
	     (assign student nil))
	(a15 check in
	     (cond nil (studentp student))
	     (do (assign name (robot-whois student))
		 (start-student-record)))
	(a7  in   out
	     (notify (quit (player . (@ student))))
	     (save-student-record))

	(a9  active active (cond (notify ((- help) (player . (@ student))))
				 (<= (idle active) idlemax)))
;;	(a8  active idle   (cond nil (> (idle active) idlemax))
	(a8  active idle   (cond nil (and (not (in working)) (> (idle active) idlemax)))
	     (robot-mail-action "~a: Student entering idle state." name))
	(a2  active help   (cond (notify (help (player . (@ student))))
				 (<= (idle active) idlemax)) (assign tick 0))

	(a3  help help     (cond (notify (help (player . (@ student))))
				 (<= tick confused))
	     (assign tick (+ tick 1)))
	(a4  help active   (notify ((- help) (player . (@ student)))))
	(a5  help confused (cond (not (notify *)) (> tick confused))
	     (robot-mail-action "~a: Student entering confused state." name))
	(a6  help idle     (cond (not (notify (help)))
				 (and (<= tick confused) (> (idle help) idlemax))))

;	(a19  idle active  (notify (* (player . (@ student)))))
	(a19  idle active  (or (notify (* (player . (@ student)))) (cond nil (in working))))

	(a11 confused confused (notify (help (player . (@ student)))) (assign tick (+ tick 1)))
	(a12 confused active   (notify ((- help) (player . (@ student))))
	     (robot-mail-action "~a: Student leaving confused state." name))
	(a13 confused idle (and (not (notify *)) (timeout 30 *))
	     (robot-mail-action "~a: Confused student is now idle." name))

	(b1  neutral working
	     (notify (one-of new-history-name new-history change-history-name)))
	(b2  create setup
	     (notify (one-of get-maximum-duration set-history-duration
			     how-many-prop-sets get-prop-set-info
			     get-prop-info get-prop-param-info
			     select-prop get-initial-value change-initial-value
			     type-of-strex get-strex-min get-strex-max get-strex-default
			     get-strex-units get-strex-in get-strex-out
			     get-strex-owner get-strex-min)))
	(b3 setup run
	    (notify (one-of check-history-status run-history)))
	(b4 run checkresult
	    (notify (one-of find-questions-for find-answer get-global-summary
			    how-many-monitored how-many-datapoints transmit-datapoints
			    user-nvalue-query generate-report)))
	(b5  checkresult setup
	     (notify (one-of get-maximum-duration set-history-duration
			     how-many-prop-sets get-prop-set-info
			     get-prop-info get-prop-param-info
			     select-prop get-initial-value change-initial-value
			     type-of-strex get-strex-min get-strex-max get-strex-default
			     get-strex-units get-strex-in get-strex-out
			     get-strex-owner get-strex-min)))

	(b6 working neutral (notify shut-down-server))
	))
	

(format t "~%Making ifd system.")
(setq ifd-system
      (initial-system ifd-nodes ifd-arcs 'student
		      :initial-alist ifd-varlist
		      :prim-events ifd-events))

(setq nodes (system-nodes ifd-system))
(setq arcs (system-arcs  ifd-system))

(setf (system-time ifd-system) 1)

(defparameter *sleep* "sleep 5")

(setq schedule
      '((2 notify (connect (player . |#3454|)))
	(4 notify (look (player . |#3454|)))
	(6 notify (say (player . |#3454|)))
	(8 notify (help (player . |#3454|)))
	(10 notify (help (player . |#3454|)))
	(12 notify (help (player . |#3454|)))
	(13 notify (help (player . |#3454|)))
	(14 notify (help (player . |#3454|)))
	(15 notify (help (player . |#3454|)))
	(18 notify (say (player . |#3454|)))
	(20 notify (execute (player . |#3454|)))
	(50 notify (execute (player . |#3454|)))
	(52 notify (disconnect (player . |#3454|)))
	(54 . halt)))

;; (setf (system-schedule ifd-system) schedule)

(format t "~%Checking the IFD System:")
	
(systemp ifd-system)
