
;; ------------------------------------------------------------
;; Michael K. Smith.
;; Copyright 1996 Computational Logic, Inc. All Rights Reserved
;; ------------------------------------------------------------

;; State machine responds to event notifications from MOO.

;; (robot-eval string) - Supports sending a form to the avatar for evaluat
;; (robot-mail to subject msg) - Supports sending Unix email.  All args are strings.

(require "mud-client")

(provide "robot")

;; This defines reload and ireload to load all the other object or
;; lisp files respectively, other than robot.

(defvar *robot-compiled* nil)

(defvar *statechart* "Student+sim.lisp")
(defvar *teachers-assistant* "mksmith@cli.com")

(defvar *step-wait-interval* 5)
(defvar *eval-wait-interval* 1)

(defvar *robot-model* 'ifd-system)      

(defparameter *step-wait* (format nil "sleep ~d" *step-wait-interval*))
(defparameter *eval-wait* (format nil "sleep ~d" *eval-wait-interval*))

(defvar *start-time* 0)

(defvar *system* nil)

;; ----------------------------------------------------------------------
;; Robot step

;; The system argument should be a previously defined statechart.

(defun robot (&optional (system *robot-model*))
  (if (boundp system)
      (progn (format t "~%Entering Robot.~%")
	     (setq *start-time* (get-universal-time))
	     (setq *system* (initial-system-step  (eval system)))
	     (tk::start-tk)
	     (if (connect-to-mud)
		 (progn (run-robot)
			(exit-mud)))
	     (format t "~%Exiting Robot.~%"))
      (format t "~%No system named ~a.~%" system)))

(defvar counter 0)

(defun run-robot ()
  (mcp-clear)
  (iterate while (and (robot-step)
		      (not (system-halted *system*))))
  (robot-cleanup *system*))

(defvar *whitespace* '(#\Space #\Newline #\Tab #\Backspace #\Page #\Return #\Rubout))

(defun real-listen (&optional (channel *standard-input*))
  ;; If T, we have NEW input to read.
  (if (listen channel)
      (if (member (peek-char nil channel) *whitespace* :test #'char-equal)
	  (progn (read-char channel) (real-listen channel))
	  t)
      (mcp-file-listen)))

(defun robot-step (&aux new)
  (or (real-listen t) (step-wait))
  (setq new
	(catch 'exit
	  (cond ((listen t)
		 (let ((event (ignore-errors (read))))
		   (cond ((member event '(quit exit bye halt (bye)))
			  (throw 'exit 'quit))
			 ((not (consp event))
			  (queue-event (cons 0 event) *system*))
			 ((and (eq (car event) 'notify)
			       (consp (cdr event))
			       (stringp (cadr event)))
			  (setq event (list (car event) (mcp-parse (cadr event)))))
			 ((eq (car event) 'moo)
			  (send-to-mud (format nil "~{~a ~}" (cdr event))))
			 (t (queue-event (cons 0 event) *system*)))))
		((mcp-listen)
		 (let ((event (mcp-read-line)))
		   (if (and event (stringp event))
		       (progn (tk::insert-mcp event)
			      (tk::insert-mcp *mcp-newline*)
			      (setq event (list 'notify (mcp-parse event)))))
		   (cond ((member event '(quit exit bye halt)) (throw 'exit 'quit))
			 ((equal event 't) nil)
			 (t (queue-event (cons 0 event) *system*)))))
		(t nil))
	  (if (> counter 2)
	      (progn (setq counter 0)
	      (status-report "~%[~4d (~6d seconds)] ~s"
			 (system-time *system*)
			 (- (get-universal-time) *start-time*)
			 (system-configuration *system*)))
	      (setq counter (+ counter 1)))))
  (if (equal new 'quit)
      nil
      (setq *system* (step-system *system*))))

(defun step-wait () (sleep *step-wait-interval*))

(defun eval-wait () (sleep *eval-wait-interval*))

(defun robot-cleanup (s) s)

(defun restart ()
  (if (or (not (boundp '*system*)) (not (systemp *system*)))
      (format t "*SYSTEM* is not currently defined as a system.")
      (progn
	(format t "~%Re-entering Robot.~%")
	(run-robot)
	(format t "~%Exiting Robot.~%"))))

(defun notify (&rest l)
  (if (and (consp l) (stringp (car l)))
      (let ((msg (mcp-parse (car l))))
	(format t "~%NOTIFY (~s ~{~s~})~%" (car msg) (cdr msg)))
      (format t "~%~%NOTIFY received out of Robot loop.~%~%")))

;; Send a form to the avatar for evaluation.

(defun robot-eval (form)
  "May fail for several reasons.  1. If what we read is not an
avatar_eval response. 2. If the first thing we read from
the mud doesn't match FORM. 3. If we timeout."
  (let ((value nil))
    (if (not (stringp form)) (setq form (format nil "~s" form)))
    (send-to-mud "; me:mcp_loop()")
    (send-to-mud (format nil "###avatar_eval expr: ~s" form))
    (send-to-mud "###avatar_endloop")
    (iterate for tick from 0 to 5
	     while (not value) 
	     do (if (mcp-listen)
		    (let ((result (mcp-read-line)))
		      (cond ((and result (stringp result))
			     (let ((results (mcp-parse result)))
			       (if (and (consp results)
					(eq (car results) 'avatar_eval)
					(assoc 'result (cdr results))
					(equal (cdr (assoc 'expr (cdr results))) form))
				   (setq value (cdr (assoc 'result (cdr results))))
				   nil)))
			    (t nil)))
		    (eval-wait)))
    (cond ((not value) nil)
	  ((and (stringp value) 
		(> (length value) 0)
		(char-equal (aref value 0) #\())
	   (cadr (mcp-read value)))
	  ((consp value)
	   (eq (car value) 1) (cadr value) nil)
	  (t value))))

(defun robot-mail (to subject message)
  ;; Send Unix email.
  (if (not (and (stringp to) (stringp subject) (stringp message)))
      (format t "~%Failed to send mail.  Ill-formed args.")
      (system (format nil "mail -s ~s ~a <<EOFEOF
~a
EOFEOF
" subject to message))))

(defparameter *teacher* "Alice")

(defun robot-mail-action (msg args)
  ;; ARGS should be 
  (send-to-mud (format nil "page ~a ~a"
		       *teacher* (format nil msg args)))
  (robot-mail *teachers-assistant* (format nil msg args)
	      (format nil msg args)))

(defun studentp (x)
  ;; If we can't get an answer, assume not.
  (if (and x (symbolp x) (char-equal (aref (symbol-name x) 0) #\#))
      (let ((result (robot-eval (format nil "parent(~a)==$student" x))))
	(if (eq result 1)
	    t 
	    nil))))

(defun avatarp (x)
  ;; If we can't get an answer, assume not.
  (if (and x (symbolp x) (char-equal (aref (symbol-name x) 0) #\#))
      (let ((result (robot-eval (format nil "parent(~a)==$avatar" x))))
	(if (eq result 1)
	    t 
	    nil))))

(defun programmerp (x)
  ;; If we can't get an answer, assume not.
  (if (and x (symbolp x) (char-equal (aref (symbol-name x) 0) #\#))
      (let ((result (robot-eval (format nil "parent(~a)==#53" x))))
	(if (eq result 1)
	    t 
	    nil))))

(defun robot-whois (obj)
  (if (and obj (symbolp obj) (char-equal (aref (symbol-name obj) 0) #\#))
      (robot-eval (format nil "~a.name" obj))))

(defun robot-room ()
  (robot-eval "#15328.location"))

(defun robot-find-student ()
  (let ((room (robot-room)))
    (if room
	(let ((objs (robot-eval (format nil "~a.contents" room))))
	  (iterate for x in objs
		   when (student x)
		   return x)))))

(defun robot-find-avatar ()
  (let ((room (robot-room)))
    (if room
	(let ((objs (robot-eval (format nil "~a.contents" room))))
	  (iterate for x in objs
		   when (avatarp x)
		   return x)))))

(defun robot-find-mikes ()
  (let ((room (robot-room)))
    (if room
	(let ((objs (robot-eval (format nil "~a.contents" room))))
	  (iterate for x in objs
		   when (string-equal (robot-whois x) "MikeS")
		   return x)))))

(defparameter *test-log* "test.log2")

(defun test-robot (&optional (system *robot-model*))
  (if (boundp system)
      (progn (format t "~%Entering Robot.~%")
	     (setq *start-time* (get-universal-time))
	     (setq *system* (initial-system-step  (eval system)))
	     ;; (connect-to-mud)
	     (tk::start-tk)
	     (mud-initialize-log)
	     (with-open-file (*mcp-input* *test-log* :direction :input) 
			     (run-robot))
	     ;; (exit-mud)
	     (close *mud-log*)
	     (format t "~%Exiting Robot.~%"))
      (format t "~%No system named ~a.~%" system)))

